<?php
/**
* Nom du fichier	SQRCLookup.php
* @author Alain Hotte
* @author Marie-Eve Levesque <marieeve_83@hotmail.com>
*
* Historique des modifications :
* 2006-09-05	Creation de la classe a partir des fonctions Geo2Sqrc et Sqrc2Geo de Alain Hotte
* 				Et ajout des fonctions ___2___
* 2006-09-06	Ajout arrDec2xy et arrXy2dec
* 				Modif xy2dec et xy2deg (inverser latitude et longitude dans array retourne)
* 2006-08-07	Ajout GetCapabilities GetSQRC
*
* Description de la classe :
* Permet de retourner le nom d'un feuillet SQRC a partir d'une coordonnee geographique, 
* et vice-versa (retourner la geometrie d'un feuillet a partir de son nom)
**/
class SQRCLookup
{
	//protected $epsg = "";
	//const VERSION_ACTUELLE = "1.0.0";
	
	/**
	 * 
	 *
	 */
	function __construct($epsg="32188") {
		$this->epsg=$epsg;
	}
	
	/**
	* Fonction Geo2Sqrc
	* But	:	Fonction transformant une coordonnee geographique 
	* 			en un nom de fichier SQRC pour une echelle donnee
	* 			entre 1:200 et 1:50000.
	* Parametres:
	* 	Entree	:	fPhi
	*							fLam
	*							iEchelle
	*
	*		Sortie	:	sReponse
								Une string de 14 caracteres etant un nom
	*							reconnu de feuillet SQRC. Dans le cas de 
	*							l'echelle 1:200, la string a 16 caracteres.
	*							l'echelle 1:50000, la string a 5 caracteres
	*
	* 						NOTE IMPORTANTE: Si un des parametre en entree
	*							est en erreur, alors la string retournee sera vide.
	*
	* Variables locales
	* 	Nom			Description
	*		---				-----------
	*	fPhi			Latitude du point de reference.
	*
	*	fLam			Longitude du point de reference.
	*
	*	iEchelle	Echelle du feuillet SQRC demande.
	*      			Les valeurs possibles sont: 
	*						200, 500, 1000, 2000, 5000, 10000, 20000 et 50000
	*
	*	fPhi0			Variable de calcul.
	*	fLambda0	Variable de calcul.
	*	aNiv2			Array de valeurs fixes.
	*	aNiv3			Array de valeurs fixes.
	*	iY1				Variable de travail.
	*	iX1				Variable de travail.
	*	iY2				Variable de travail.
	*	iX2				Variable de travail.
	*	iY3				Variable de travail.
	*	iX3				Variable de travail.
	*	iY4				Variable de travail.
	*	iX4				Variable de travail.
	*	sS1				Premiere partie du code de feuillet SQRC.
	*	sS2				Deuxieme partie du code de feuillet SQRC.
	*	sS3				Troisieme partie du code de feuillet SQRC.
	*	sS4				Quatrieme partie du code de feuillet SQRC.
	*	sS5				Cinquieme partie du code de feuillet SQRC.
	* sReponse	Resultat retourne par la fonction
	* iDebug		Indicateur de niveau pour debug
	* aSqrcScales Array contenant les echelles permises pour les decoupages SQRC
	*	
	* Variables globales
	* 	Nom		Description
	*	---		-----------
	*	Aucune
	*
	* Fonctions/Programmes appelles
	* 	Nom		Description
	*	---		-----------
	*	Aucune
	*
	*****************************************************************
	* Ecrit par		: Alain Hotte a.g.
	* De					: CMM
	* Version			: 1.0
	* Date				: 16 aout 2006
	*/
	function Geo2Sqrc($fPhi,$fLambda,$iEchelle) 
	{
	
		//Initialisation de variables locales
		$iDebug = 1;
		$sReponse = "";
		$aSqrcScales = array("200","500","1000","2000","5000","10000","20000","50000");
		$fPhi0 = 44.0;
		$fLambda0 = 56.0;
	
		$aNiv2 = array();
		$aNiv2[0] = array("A","B","C","D");
		$aNiv2[1] = array("H","G","F","E");
		$aNiv2[2] = array("I","J","K","L");
		$aNiv2[3] = array("P","O","N","M");
		
		$aNiv3 = array();
		$aNiv3[0] = array("01","02","03","04");
		$aNiv3[1] = array("08","07","06","05");
		$aNiv3[2] = array("09","10","11","12");
		$aNiv3[3] = array("16","15","14","13");
		
		$iX1 = 0;
		$iX2 = 0;
		$iX3 = 0;
		$iX4 = 0;
		$iY1 = 0;
		$iY2 = 0;
		$iY3 = 0;
		$iY4 = 0;
		$sS1 = "";
		$sS2 = "";
		$sS3 = "";
		$sS4 = "";
		$sS4 = "";
		$sS5 = "";
	
		// Validation des parametres d'entre
		if ($fPhi < 44 || $fPhi > 64 || $fLambda < 56 || $fLambda > 80) {
			// Position hors limite du quebec
			if ($iDebug == 1) {
				echo "Position hors limite du quebec\n";
				echo "Phi: $fPhi\n";
				echo "Lambda: $fLambda\n";
			}
			return $sReponse;
		}
		
		if (!in_array($iEchelle,$aSqrcScales)) {
			// Echelle non valide
			if ($iDebug == 1) {
				echo "Echelle non valide\n";
				echo "Echelle: $iEchelle\n";
			}
			return $sReponse;
			
		}
		
		// Calculs du numero de feuilllet 1:1000000 (1er niveau)
		$iY1 = floor(($fPhi - $fPhi0) / 4.0);
		$iX1 = floor(($fLambda - $fLambda0) / 8.0);
		$sS1 = (string) ((10 * ($iX1 + 1)) + ($iY1 + 1));
	
		// Reinitialisation de fPhi0 et fLambda0 pour le feuillet 1:1000000
		$fPhi0 = $fPhi0 + (4.0 * $iY1);
		$fLambda0 = $fLambda0 + (8.0 * $iX1); 
	
		// Calculs du numero de feuilllet 1:250000 (2e niveau)
		$iY2 = floor($fPhi - $fPhi0);
		$iX2 =  floor(($fLambda - $fLambda0) / 2.0);
		$sS2 = (string) $aNiv2[$iY2][$iX2];
	
		// Reinitialisation de fPhi0 et fLambda0 pour le feuillet 1:250000
		$fPhi0 = $fPhi0 + $iY2;
		$fLambda0 = $fLambda0 + (2.0 * $iX2); 
	
		// Calculs du numero de feuilllet 1:50000 (3e niveau)
		$iY3 = floor(($fPhi - $fPhi0) * 4);
		$iX3 = floor(($fLambda - $fLambda0) * 2);
		$sS3 = (string) $aNiv3[$iY3][$iX3];
	
		// Reinitialisation de fPhi0 et fLambda0 pour le feuillet 1:50000
		$fPhi0 = $fPhi0 + ($iY3 / 4);
		$fLambda0 = $fLambda0 + ($iX3 / 2); 
	
		// Traitement particulier pour chaque echelle
		
		switch($iEchelle) {
		case "50000":
			$sReponse = $sS1 . $sS2 . $sS3;
			break;
		case "20000":
			$sS4 = "200";
			$iY4 = floor(($fPhi - $fPhi0) * 8.0) + 1;
			$iX4 = 2 - floor(($fLambda - $fLambda0) * 4.0);
			$sS5 = "0" . (string) $iY4 . "0" . (string) $iX4;
			$sReponse = $sS1 . $sS2 . $sS3 . "-" . $sS4 . "-" . $sS5;
			break;
		case "10000":
			$sS4 = "100";
			$iY4 = floor(($fPhi - $fPhi0) * 16.0) + 1;
			$iX4 = 4 - floor(($fLambda - $fLambda0) * 8.0);
			$sS5 = "0" . (string) $iY4 . "0" . (string) $iX4;
			$sReponse = $sS1 . $sS2 . $sS3 . "-" . $sS4 . "-" . $sS5;
			break;
		case "5000":
			$sS4 = "050";
			$iY4 = floor(($fPhi - $fPhi0) * 32.0) + 1;
			$iX4 = 8 - floor(($fLambda - $fLambda0) * 16.0);
			$sS5 = "0" . (string) $iY4 . "0" . (string) $iX4;
			$sReponse = $sS1 . $sS2 . $sS3 . "-" . $sS4 . "-" . $sS5;
			break;
		case "2000":
			$sS4 = "020";
			$iY4 = floor(($fPhi - $fPhi0) * 80.0) + 1;
			$iX4 = 20 - floor(($fLambda - $fLambda0) * 40.0);
			if ($iY4 < 10) 
				$sS5 = "0" . (string) $iY4;
			else
				$sS5 = (string) $iY4;
			if ($iX4 < 10)
				$sS5 = $sS5 . "0" . (string) $iX4;
			else
				$sS5 = $sS5 . (string) $iX4;
			$sReponse = $sS1 . $sS2 . $sS3 . "-" . $sS4 . "-" . $sS5;
			break;
		case "1000":
			$sS4 = "010";
			$iY4 = floor(($fPhi - $fPhi0) * 160.0) + 1;
			$iX4 = 40 - floor(($fLambda - $fLambda0) * 80.0);
			if ($iY4 < 10)
				$sS5 = "0" . (string) $iY4;
			else
				$sS5 = (string) $iY4;
			if ($iX4 < 10)
				$sS5 = $sS5 . "0" . (string) $iX4;
			else
				$sS5 = $sS5 . (string) $iX4;
			$sReponse = $sS1 . $sS2 . $sS3 . "-" . $sS4 . "-" . $sS5;
			break;
		case "500":
			$sS4 = "005";
			$iY4 = floor(($fPhi - $fPhi0) * 320.0) + 1;
			$iX4 = 80 - floor(($fLambda - $fLambda0) * 160.0);
			if ($iY4 < 10)
				$sS5 = "0" . (string) $iY4;
			else
				$sS5 = (string) $iY4;
			if ($iX4 < 10)
				$sS5 = $sS5 . "0" . (string) $iX4;
			else
				$sS5 = $sS5 . (string) $iX4;
			$sReponse = $sS1 . $sS2 . $sS3 . "-" . $sS4 . "-" . $sS5;
			break;
		case "200":
			$sS4 = "002";
			$iY4 = floor(($fPhi - $fPhi0) * 800.0) + 1;
			$iX4 = 200 - floor(($fLambda - $fLambda0) * 400.0);
			if (iY4 < 10)
				$sS5 = "00" . (string) $iY4;
			else
				if ($iY4 > 9 && $iY4 < 100)
					$sS5 = "0" . (string) $iY4;
				else
					$sS5 = (string) $iY4;
			if ($iX4 < 10)
				$sS5 = $sS5 . "00" . (string) $iX4;
			else
				if ($iX4 > 9 && $iX4 < 100)
					$sS5 = $sS5 . "0" . (string) $iX4;
				ELSE
					$sS5 = $sS5 . (string) $iX4;
			$sReponse = $sS1 . $sS2 . $sS3 . "-" . $sS4 . "-" . $sS5;
			break;
		}
	
		return $sReponse;
	
	}
	
	
	/**
	***************************************************************
	****************************************************************
	
	 Nom	:	Sqrc2Geo
	
	 But	:	Fonction transformant un nom de feuillet SQRC en un ARRAY contenant la geometrie 
	 				(coordonnees geographiques (Latitude, Longitude)) pour le feuillet en question.
					Le ARRAY peut contenir la coordonnee du centroide, le bounding box, ou une serie de coordonnees 
					delimitant le neatline sivant le parametre passe lors de l'appel
	
	 Date de creation	: 18 aout 2006
	
	 Parametres
	 	Entree	:	sNomFeuilletSQRC,iTypeGeom,iTicInterval
	
		Sortie	:	La fonction retourne un array (aPts) (tableau a deux dimensions)
							contenant 2 colonnes (latitude, longitude) et entre 1 et xx lignes, suivant le parametre d'appel. 
							Les elements sont une suite de points (phi, lambda) decrivant le perimetre du feuillet. 
	
							Le nombre de segments varie en fonction de 
							l'echelle et de la position du feuillet.
							Les points d'inflection sont par defaut a des
							multiples precis de la position des marques
							de phi et lambda (tics) apparaissant sur les
							feuillets traces (ex: aux 6" pour le 1:1000)
	
				 			NOTE IMPORTANTE: Si un des parametre en entree
							est en erreur, alors l'array retourne sera vide.
	
	 Variables locales
	 	Nom			Description
		---			-----------
	  sNomFeuilletSQRC	Le nom d'un feuillet SQRC valide, sous forme de string.  
											La string a 14 caracteres de long pour les echelles 20000, 10000, 5000,  2000, 1000, 500 
											et a la forme 99A99-999-9999. 
											Pour	l'echelle 200, la string a 16 caracteres de long et a la forme 99A99-999-999999. 
	       							Pour l'echelle 50000, la string a 5 caracteres de long et � la forme 99A99.
		aNiv1							Array de valeurs fixes.
		aNiv2							Array de valeurs fixes.
		aNiv3							Array de valeurs fixes.
		iNb								Nombre de champs dans sNomFeuilletSQRC.
		aChampsSQRC				sNomFeuilletSQRC decompose.
		fPhi0							Variable de travail.
		fLambda0					Variable de calcul.
		bFound						Booleen indiquant qui on a trouve les
											valeurs dans les tables aNiv2 et aNiv3.
		iX2								Variable de boucle.
		iY2								Variable de boucle.
		sS1								Variable de travail.
		sS2								Variable de travail.
		sS3								Variable de travail.
		iX3								Variable de boucle.
		iY3								Variable de boucle.
		iEchelle					Echelle du feuillet SQRC.
	      							Les valeurs possibles sont: 
											200, 500, 1000, 2000, 5000, 10000, 20000 et 50000
		fIncrement				Variable de calcul.
		fPhi1							Variable de calcul.
		fLambda1					Variable de calcul.
		aLimites					Array de valeurs de calcul.
		fPhi_cal					Variable de calcul.
		fLambda_cal				Variable de calcul.
		iIdx							Variable d'indexation d'array.
		iIdx1							Variable de boucle.
		iIdx2							Variable de boucle.
		aPts							Array de valeurs retournees
	 	bEchelle50000			Booleen indiquant si c'est l'�chelle 50000
	
	 Variables globales
	 	Nom		Description
		---		-----------
		Aucune
	
	 Fonctions/Programmes appelles
	 	Nom		Description
		---		-----------
		Aucune
	
	****************************************************************
	 Ecrit par: 				Alain Hotte a.g.
	 De: 								Communaute Metropolitaine de Montreal
	
	****************************************************************
	****************************************************************
	*/
	function Sqrc2Geo($sNomFeuilletSQRC,$iTypeGeom,$iTicInterval="") 
	{
	
		//Initialisation de variables locales
		$iDebug = 0;
		$aPts = "";
		$aSqrcScales = array("200","500","1000","2000","5000","10000","20000","50000");
		$fPhi0 = 44.0;
		$fLambda0 = 56.0;
		$aNiv1 = array("11","12","13","14","15","21","22","23","24","25","31","32","33","34","35");
	
		$aNiv2 = array();
		$aNiv2[0] = array("A","B","C","D");
		$aNiv2[1] = array("H","G","F","E");
		$aNiv2[2] = array("I","J","K","L");
		$aNiv2[3] = array("P","O","N","M");
	
		$aNiv3 = array();
		$aNiv3[0] = array("01","02","03","04");
		$aNiv3[1] = array("08","07","06","05");
		$aNiv3[2] = array("09","10","11","12");
		$aNiv3[3] = array("16","15","14","13");
	
		$aChampsSQRC = explode("-",$sNomFeuilletSQRC);
		$iNb = sizeof($aChampsSQRC);
		
		$bEchelle50000 = FALSE;
		
		// Validation de la structure de separateurs du numero de feuillet en 3 parties, ou 1 partie pour l'echelle 50000 
		if ($iNb != 3 && $iNb != 1) {
			// mauvaise structure pour le nom de feuillet SQRC
			if ($iDebug == 1) {
				echo "Mauvaise structure pour le nom de feuillet SQRC\n";
				echo "Nom de feuillet: $sNomFeuilletSQRC\n";
			}
			return $aPts;
		}
	
		// Validation et traitement des deux premiers caracteres de la premiere partie du numero de feuillet (1:1000000)
		$sS1 = (int) substr($aChampsSQRC[0],0,2);
		$iY1 = (int) substr($aChampsSQRC[0],1,1) - 1;
		$iX1 = (int) substr($aChampsSQRC[0],0,1) - 1;	
		if (in_array($sS1,$aNiv1)) {
			$fPhi0 = $fPhi0 + (4.0 * $iY1);
			$fLambda0 = $fLambda0 + (8.0 * $iX1);
		}
		else {
			// feuillet 1000000 non valide
			if ($iDebug == 1) {
				echo "Feuillet 1000000 non valide\n";
				echo "sS1: '$sS1'\n";
				print_r($aNiv1);
			}
			return $aPts;	
		}
	
		// Validation et traitement du troisieme caractere de la premiere partie du numero de feuillet (1:250000)
		$bFound = FALSE;
		$sS2 = strtoupper(substr($aChampsSQRC[0],2,1));
		for ($iX2 = 0; $iX2 < 4; $iX2++) {
			for ($iY2 = 0; $iY2 < 4; $iY2++) {
				if ($aNiv2[$iY2][$iX2] == $sS2) {
					$fPhi0 = $fPhi0 + $iY2;
					$fLambda0 = $fLambda0 + (2.0 * $iX2);
					$bFound = TRUE;
				}
			}
		}
		if (! $bFound) {
			// feuillet 250000 non valide
			if ($iDebug == 1) {
				echo "Feuillet 250000 non valide\n";
				echo "sS2: '$sS2'\n";
				print_r($aNiv2);
			}
			return $aPts;
		}
			
		// Validation et traitement des caracteres quatre et cinq de la premiere partie du numero de feuillet (1:50000)
		$bFound = FALSE;
		$sS3 = (substr($aChampsSQRC[0],3,2));
		for ($iX3 = 0; $iX3 < 4; $iX3++) {
			for ($iY3 = 0; $iY3 < 4; $iY3++) {
				if ($aNiv3[$iY3][$iX3] == $sS3) {
					$fPhi0 = $fPhi0 + ($iY3 / 4.0);
					$fLambda0 = $fLambda0 + ($iX3 / 2.0);
					$bFound = TRUE;
				}
			}
		}
		if (! $bFound) {
			// feuillet 50000 non valide
			if ($iDebug == 1) {
				echo "Feuillet 50000 non valide\n";
				echo "sS3: '$sS3'\n";
				print_r($aNiv3);		
			}
			return $aPts;
		}
	
		// verifie si c'est l'echelle 50000 
		if ($iNb == 1) {
			$bEchelle50000 = TRUE;
			$iEchelle = 50000;
		}
		
		// Validation de la deuxieme partie du numero de feuillet (echelle) si different de 50000
		if ($iNb == 3) {
			$iEchelle = (int) $aChampsSQRC[1] * 100;
			if (!in_array($iEchelle,$aSqrcScales)) {
				// echelle non valide
				if ($iDebug == 1) {
					echo "echelle non valide\n";
					echo "iEchelle: '$iEchelle'\n";
					print_r($aSqrcScales);		
				}
				return $aPts;
			}
		}
	
		// Transformation du point de base en secondes d'arc au lieu d'en degre decimal pour ficiliter les calculs		
		$fPhi0 = $fPhi0 * 3600.0;
		$fLambda0 = $fLambda0 * 3600.0;
		
		// Validation de la troisieme partie du numero de feuillet et
		// traitement particulier pour chaque echelle 
	
		switch($iEchelle) {
		case "50000":
			$fIncrement = 300.0;  // 5 min
			$fPhi1 = $fPhi0 + 900.0; // 15 min 
			$fLambda1 = $fLambda0 + 1800.0; //30 min
			break;
				
		case "20000":
			if ((int) substr($aChampsSQRC[2],0,2) > 0 && (int) substr($aChampsSQRC[2],0,2) < 3 && (int) substr($aChampsSQRC[2],2,2) > 0 && (int) substr($aChampsSQRC[2],2,2) < 3) {
				$fIncrement = 120.0; // 2 min
				$fPhi0 = $fPhi0 + (((int) substr($aChampsSQRC[2],0,2) - 1) * 450.0);
				$fPhi1 = $fPhi0 + 450.0; // 7.5 min
				$fLambda0 = $fLambda0 + ((2 - (int) substr($aChampsSQRC[2],2,2)) * 900.0);
				$fLambda1 = $fLambda0 + 900.0; // 15 min
			}
			else {
			// feuillet 20000 non valide
				if ($iDebug == 1) {
					echo "Feuillet 20000 non valide\n";
					echo "sNomFeuilletSQRC: '$sNomFeuilletSQRC'\n";
				}
				return $aPts;	
			}
			break;
	
		case "10000":
			if ((int) substr($aChampsSQRC[2],0,2) > 0 && (int) substr($aChampsSQRC[2],0,2) < 5 && (int) substr($aChampsSQRC[2],2,2) > 0 && (int) substr($aChampsSQRC[2],2,2) < 5) {
				$fIncrement = 60.0; // 1 min
				$fPhi0 = $fPhi0 + (((int) substr($aChampsSQRC[2],0,2) - 1) * 225.0);
				$fPhi1 = $fPhi0 + 225.0; // 3 min 45 sec
				$fLambda0 = $fLambda0 + ((4 - (int) substr($aChampsSQRC[2],2,2)) * 450.0);
				$fLambda1 = $fLambda0 + 450.0; // 7 min 30 sec
			}
			else {
			// feuillet 10000 non valide
				if ($iDebug == 1) {
					echo "Feuillet 10000 non valide\n";
					echo "sNomFeuilletSQRC: '$sNomFeuilletSQRC'\n";
				}
				return $aPts;	
			}
			break;
	
		case "5000":
			if ((int) substr($aChampsSQRC[2],0,2) > 0 && (int) substr($aChampsSQRC[2],0,2) < 9 && (int) substr($aChampsSQRC[2],2,2) > 0 && (int) substr($aChampsSQRC[2],2,2) < 9) {
				$fIncrement = 30.0; // 30 sec
				$fPhi0 = $fPhi0 + (((int) substr($aChampsSQRC[2],0,2) - 1) * 112.5);
				$fPhi1 = $fPhi0 + 112.5; // 1 min 52.5 sec
				$fLambda0 = $fLambda0 + ((8 - (int) substr($aChampsSQRC[2],2,2)) * 225.0);
				$fLambda1 = $fLambda0 + 225.0; // 3 min 45 sec
			}
			else {
			// feuillet 5000 non valide
				if ($iDebug == 1) {
					echo "Feuillet 5000 non valide\n";
					echo "sNomFeuilletSQRC: '$sNomFeuilletSQRC'\n";
				}
				return $aPts;	
			}
			break;
	
		case "2000":
			if ((int) substr($aChampsSQRC[2],0,2) > 0 && (int) substr($aChampsSQRC[2],0,2) < 21 && (int) substr($aChampsSQRC[2],2,2) > 0 && (int) substr($aChampsSQRC[2],2,2) < 21) {
				$fIncrement = 12.0; // 12 sec
				$fPhi0 = $fPhi0 + (((int) substr($aChampsSQRC[2],0,2) - 1) * 45);
				$fPhi1 = $fPhi0 + 45.0; // 45 sec
				$fLambda0 = $fLambda0 + ((20 - (int) substr($aChampsSQRC[2],2,2)) * 90.0);
				$fLambda1 = $fLambda0 + 90.0; // 1 min 30 sec
			}
			else {
			// feuillet 2000 non valide
				if ($iDebug == 1) {
					echo "Feuillet 2000 non valide\n";
					echo "sNomFeuilletSQRC: '$sNomFeuilletSQRC'\n";
				}
				return $aPts;	
			}
			break;
	
		case "1000":
			if ((int) substr($aChampsSQRC[2],0,2) > 0 && (int) substr($aChampsSQRC[2],0,2) < 41 && (int) substr($aChampsSQRC[2],2,2) > 0 && (int) substr($aChampsSQRC[2],2,2) < 41) {
				$fIncrement = 6.0; // 6 sec
				$fPhi0 = $fPhi0 + (((int) substr($aChampsSQRC[2],0,2) - 1) * 22.5);
				$fPhi1 = $fPhi0 + 22.5; // 22.5 sec
				$fLambda0 = $fLambda0 + ((40 - (int) substr($aChampsSQRC[2],2,2)) * 45.0);
				$fLambda1 = $fLambda0 + 45.0; // 45 sec
			}
			else {
			// feuillet 1000 non valide
				if ($iDebug == 1) {
					echo "Feuillet 1000 non valide\n";
					echo "sNomFeuilletSQRC: '$sNomFeuilletSQRC'\n";
				}
				return $aPts;	
			}
			break;
	
		case "500":
			if ((int) substr($aChampsSQRC[2],0,2) > 0 && (int) substr($aChampsSQRC[2],0,2) < 81 && (int) substr($aChampsSQRC[2],2,2) > 0 && (int) substr($aChampsSQRC[2],2,2) < 81) {
				$fIncrement = 3.0; // 3 sec
				$fPhi0 = $fPhi0 + (((int) substr($aChampsSQRC[2],0,2) - 1) * 11.25);
				$fPhi1 = $fPhi0 + 11.25; // 11.25 sec
				$fLambda0 = $fLambda0 + ((80 - (int) substr($aChampsSQRC[2],2,2)) * 22.5);
				$fLambda1 = $fLambda0 + 22.5; // 22.5 sec
			}
			else {
			// feuillet 500 non valide
				if ($iDebug == 1) {
					echo "Feuillet 500 non valide\n";
					echo "sNomFeuilletSQRC: '$sNomFeuilletSQRC'\n";
				}
				return $aPts;	
			}
			break;
	
		case "200":
			if ((int) substr($aChampsSQRC[2],0,3) > 0 && (int) substr($aChampsSQRC[2],0,3) < 201 && (int) substr($aChampsSQRC[2],3,3) > 0 && (int) substr($aChampsSQRC[2],3,3) < 201) {
				$fIncrement = 1.0; // 1 sec
				$fPhi0 = $fPhi0 + (((int) substr($aChampsSQRC[2],0,3) - 1) * 4.5);
				$fPhi1 = $fPhi0 + 4.5; // 4.5 sec
				$fLambda0 = $fLambda0 + ((200 - (int) substr($aChampsSQRC[2],3,3)) * 9.0);
				$fLambda1 = $fLambda0 + 9.0; // 9 sec
			}
			else {
			// feuillet 200 non valide
				if ($iDebug == 1) {
					echo "Feuillet 200 non valide\n";
					echo "sNomFeuilletSQRC: '$sNomFeuilletSQRC'\n";
				}
				return $aPts;	
			}
			break;
	
		}
	
		switch ($iTypeGeom) {
			case "1":
				if ($iDebug == 1) {
					echo "Geometrie type 1: Centroide\n";
					echo "fPhi0: $fPhi0\tfPhi1: $fPhi1\tfLambda0: $fLambda0\tfLambda1: $fLambda1\n";
				}
				$fPhi_c = ((($fPhi0 + $fPhi1) / 2.0) / 3600.0);
				$fLambda_c = ((($fLambda0 + $fLambda1) / 2.0) / 3600.0);
				$aPts = ARRAY($fPhi_c,$fLambda_c);
				return $aPts;
			break;
		
			case "2":
				if ($iDebug == 1) {
					echo "Geometrie type 2: Bounding box\n";
					echo "fPhi0: $fPhi0\tfPhi1: $fPhi1\tfLambda0: $fLambda0\tfLambda1: $fLambda1\n";
				}
				$aPts = array();
				$aPts[0] = array($fPhi0 / 3600.0,$fLambda0 / 3600.0);
				$aPts[1] = array($fPhi0 / 3600.0,$fLambda1 / 3600.0);
				$aPts[2] = array($fPhi1 / 3600.0,$fLambda1 / 3600.0);
				$aPts[3] = array($fPhi1 / 3600.0,$fLambda0 / 3600.0);
				$aPts[4] = array($fPhi0 / 3600.0,$fLambda0 / 3600.0);
				return $aPts;		
			break;
					
			case "3":
				if ($iDebug == 1) {
					echo "Geometrie type 3: NeatLine\n";
					echo "fIncrement: $fIncrement\n";
				}
				$aPts = array();
				$iIdx = 0;
	
				// Traitement du bas du cadre
				$aPts[$iIdx] = array($fPhi0 / 3600.0,$fLambda0 / 3600.0);
				$iIdx++;
				$fLambda_cal = ((floor($fLambda0 / $fIncrement) + 1) * $fIncrement);
				while ($fLambda_cal < $fLambda1) {
					$aPts[$iIdx] = array($fPhi0 / 3600.0,$fLambda_cal / 3600.0);
					$iIdx++;
					$fLambda_cal = $fLambda_cal + $fIncrement;
				}
	
				// Traitement du cote gauche du cadre
				$aPts[$iIdx] = array($fPhi0 / 3600.0,$fLambda1 / 3600.0);
				$iIdx++;
				$fPhi_cal = ((floor($fPhi0 / $fIncrement) + 1) * $fIncrement);
				while ($fPhi_cal < $fPhi1) {
					$aPts[$iIdx] = array($fPhi_cal / 3600.0,$fLambda1 / 3600.0);
					$iIdx++;
					$fPhi_cal = $fPhi_cal + $fIncrement;
				}
				
				// Traitement du haut du cadre
				$aPts[$iIdx] = array($fPhi1 / 3600.0,$fLambda1 / 3600.0);
				$iIdx++;
				$fLambda_cal = (floor($fLambda1 / $fIncrement) * $fIncrement);
				if ($fLambda_cal == $fLambda1) {
					$fLambda_cal = $fLambda_cal - $fIncrement;
				}
				while ($fLambda_cal > $fLambda0) {
					$aPts[$iIdx] = array($fPhi1 / 3600.0,$fLambda_cal / 3600.0);
					$iIdx++;
					$fLambda_cal = $fLambda_cal - $fIncrement;
				}
				
				// Traitement du cote droit du cadre
				$aPts[$iIdx] = array($fPhi1 / 3600.0,$fLambda0 / 3600.0);
				$iIdx++;
				$fPhi_cal = (floor($fPhi1 / $fIncrement) * $fIncrement);
				if ($fPhi_cal == $fPhi1) {
					$fPhi_cal = $fPhi_cal - $fIncrement;
				}
				while ($fPhi_cal > $fPhi0) {
					$aPts[$iIdx] = array($fPhi_cal / 3600.0,$fLambda0 / 3600.0);
					$iIdx++;
					$fPhi_cal = $fPhi_cal - $fIncrement;
				}
	
				// Repetition du point initial
				$aPts[$iIdx] = array($fPhi0 / 3600.0,$fLambda0 / 3600.0);
	
				return $aPts;
			break;
			
			default:
				echo "Type de geometrie invalide: $iTypeGeom\n";
				return $aPts;
		}
	}
	
	
	/**
	 * Convertir une valeur degres/minutes/secondes en valeur decimale
	 *
	 * @param String $deg Valeur de type 45°53'36''
	 * @return String Valeur decimale equivalente
	 */
	function deg2dec($deg) {
		// Replacer les symboles de degres, minutes et secondes par des points
		$deg=str_replace("°",",",$deg);
		$deg=str_replace("'",",",$deg);
		
		$infosDeg=split(",",$deg);
		if($infosDeg[0]<0) {
			return $infosDeg[0]-($infosDeg[1]/60)-($infosDeg[2]/3600);
		} else {
			return $infosDeg[0]+($infosDeg[1]/60)+($infosDeg[2]/3600);
		}
	}

	/**
	 * Convertir une valeur decimale en valeur degres/minutes/secondes 
	 *
	 * @param String $deg Valeur decimale
	 * @return String Valeur de type 45°53'36'' equivalente
	 */
	function dec2deg($dec) {
		$infosDeg=split("\.",$dec);
		$degres=$infosDeg[0];
		
		$infosDeg=split("\.",((".".$infosDeg[1])*60)); // On doit ajouter un . avant la valeur restante, avant de multiplier par 60
		$minutes=$infosDeg[0];
		
		$infosDeg=split("\.",((".".$infosDeg[1])*60)); // On doit ajouter un . avant la valeur restante, avant de multiplier par 60
		$secondes=$infosDeg[0];
		
		return $degres."°".$minutes."'".$secondes."''";
	}

	/**
	 * Convertir une latitude et longitude de valeur decimale en valeur x y
	 *
	 * @param String $lat Latitude en degres decimaux
	 * @param String $long Longitude en degres decimaux
	 * @param String $this->epsg Code du systeme epsg
	 * @return Array Valeurs pour x et y
	 */
	function dec2xy($long,$lat) {
		$projInObj=ms_newProjectionObj("proj=latlong");
		$projOutObj=ms_newProjectionObj("init=epsg:".$this->epsg);
		
		$noPoint=ms_newpointobj();
		$noPoint->setXY($long,$lat);
		
		$noPoint->project($projInObj,$projOutObj);
		return array("x"=>$noPoint->x,"y"=>$noPoint->y);
	}
	
	/**
	 * Convertir une latitude et longitude de valeur degres/minutes/secondes en valeur x y
	 *
	 * @param String $lat Latitude en degres/minutes/secondes
	 * @param String $long Longitude en degres/minutes/secondes
	 * @param String $this->epsg Code du systeme epsg
	 * @return Array Valeurs pour x et y
	 */
	function deg2xy($long,$lat) {
		return $this->dec2xy($this->deg2dec($long),$this->deg2dec($lat));
	}
	
	/**
	 * Convertir une valeur x y en latitude et longitude de valeur decimale
	 *
	 * @param String $x Valeur en x
	 * @param String $y Valeur en y
	 * @param String $this->epsg Code du systeme epsg
	 * @return Array Valeurs pour latitude et longitude (en degres decimaux)
	 */
	function xy2dec($x,$y) {
		$projInObj=ms_newProjectionObj("init=epsg:".$this->epsg);
		$projOutObj=ms_newProjectionObj("proj=latlong");
		
		$noPoint=ms_newpointobj();
		$noPoint->setXY($x,$y);
		
		$noPoint->project($projInObj,$projOutObj);
		return array("long"=>$noPoint->x,"lat"=>$noPoint->y);
	}
	
	/**
	 * Convertir une valeur x y en latitude et longitude en degres/minutes/secondes
	 *
	 * @param String $x Valeur en x
	 * @param String $y Valeur en y
	 * @param String $this->epsg Code du systeme epsg
	 * @return Array Valeurs pour latitude et longitude (en degres/minutes/secondes)
	 */
	function xy2deg($x,$y) {
		$valeurs=$this->xy2dec($x,$y);
		return array("long"=>$this->dec2deg($valeurs["long"]),"lat"=>$this->dec2deg($valeurs["lat"]));
	}
	
	/**
	 * Convertir un tableau de coordonnees decimales en coordonnees XY
	 *
	 * @param array $arr Liste des coordonnees de format 
	 * 					Array(
	 * 						[0]=>Array(
	 * 							[0]=>50,
	 * 							[1]=>60
	 * 						),
	 * 						[1]=>Array(
	 * 							[0]=>50.1,
	 * 							[1]=>60.2
	 * 						)
	 * 					)
	 * 				(par exemple retourne par Sqrc2Geo)
	 * @param String $hemisLong Hemisphere pour la longitude (E ou W)
	 * @param String $hemisLat Hemisphere pour la latitude (N ou S)
	 * @return array Tableau de meme format, contenant les coordonnees converties 
	 * 				 (les indices de chaque paire de coordonnes sont x et y)
	 */
	function arrDec2xy($arr,$hemisLong,$hemisLat) {
		if(!is_array(reset($arr))) {
			if(empty($arr[0]))
				$arr[0]=$arr["long"];
			if(empty($arr[1]))
				$arr[1]=$arr["lat"];
				
			if($hemisLong=="W"&&$arr[0]>0)
				$arr[0]=-$arr[0];
			if($hemisLat=="S"&&$arr[1]>0)
				$arr[1]=-$arr[1];
			$arrModifie=$this->dec2xy($arr[0],$arr[1]);
		} else {
			foreach ($arr as $elem) {
				$arrModifie[]=$this->arrDec2xy($elem,$hemisLong,$hemisLat);
			}
		}
		return $arrModifie;
	}
	
	/**
	 * Convertir un tableau de coordonnees decimales en coordonnees XY
	 *
	 * @param array $arr Liste des coordonnees de format 
	 * 					Array(
	 * 						[0]=>Array(
	 * 							[0]=>1149331.69,
	 * 							[1]=>8385223.96
	 * 						),
	 * 						[1]=>Array(
	 * 							[0]=>1149331.69,
	 * 							[1]=>8385223.96
	 * 						)
	 * 					)
	 * 				(par exemple retourne par Sqrc2Geo)
	 * @param String $hemisLong Hemisphere pour la longitude (E ou W)
	 * @param String $hemisLat Hemisphere pour la latitude (N ou S)
	 * @return array Tableau de meme format, contenant les coordonnees converties 
	 * 				 (les indices de chaque paire de coordonnes sont long et lat)
	 */
	function arrXy2dec($arr,$hemisLong,$hemisLat) {
		if(!is_array(reset($arr))) {
			if(empty($arr[0]))
				$arr[0]=$arr["x"];
			if(empty($arr[1]))
				$arr[1]=$arr["y"];
				
			if($hemisLat=="S"&&$arr[1]>0)
				$arr[1]=-$arr[1];
			$arrModifie=$this->xy2dec($arr[0],$arr[1]);
		} else {
			foreach ($arr as $elem) {
				$arrModifie[]=$this->arrXy2dec($elem,$hemisLong,$hemisLat);
			}
		}
		return $arrModifie;
	}
	
	
	function GetCapabilities($version="1.0.0") {
		
		// Generer le XML
		
		return $xml;
	}
	
	function GetSQRC($noSqrc,$type=1,$tic,$version="1.0.0") {
		$sqrcInfos=$this->Sqrc2Geo($noSqrc,$type,$tic);
		// Generer le XML
		
		return $xml;
	}
}
?>
