<?php
/**
 * Common query utility functions
 *
 * @project     Chameleon
 * @revision    $Id: query_utils.php,v 1.3 2006/05/02 19:01:50 bartvde Exp $
 * @purpose     common utility functions used to query data
 * @author      Paul Spencer (pspencer@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

include_once( COMMON."wrapper/map_query.php" );
include_once( COMMON."xml_utils/XMLObject.php" );
include_once( COMMON."/phpwms/dbf.php");

/**
 * execute a query on the given map session object at the given location or rectangle.  If a rectangular
 * query is requested, it will only work on local layers.  WMS layers can only be queried by point.
 *
 * @param oMapSession object the map session object to query
 * @param nMinX int the x coord of the query point, or the left edge of the query region
 * @param nMinY int the y coord of the query point, or the top edge of the query region
 * @param nMaxX int optional, if > nMinX it is the right edge of the query region
 * @param nMaxY int optional, if > nMinY it is the bottom edge of the query region
 * @param bVisibilityCheck bool optional, if true, query is only executed on visible layers
 * @return array an array of values returned by the query.  For every queryable layer in
 *         the map file, the array will have a key for that layer's index ( suitable for
 *         use with $oMap->getLayer ) and either a string 'no results' or an array of
 *         results for that layer.  Index is sequential from 0, contents is another array
 *         containing key->value pairs for attribute->value from the query results
 */

function executeQuery( $oMapSession, $nMinX, $nMinY, $nMaxX = -1, $nMaxY = -1, $bVisibilityCheck = true)
{
    //this could take a while for WMS results
    @set_time_limit( 120 );

    // create a new map query object
    $oMapQuery = new MapQuery( $oMapSession );

    // initalize vars
    $aszWMSResults = array();

    // loop through all layers and add to list of layers to query
    // NOTE:  Layers will only have results generated for them if they
    // have a "template" value set.  The template value does not have
    // to be valid, it could be as simple as "ttt".
    $aszLayers = array();
    $aszTitles = array();
    $oMap = $oMapSession->oMap;
    $nCount = $oMap->numlayers;
    for ( $i=0; $i<$nCount; $i++ )
    {
        // get layer name
        $oLayer = $oMap->getLayer( $i );

        $bQueryable = false;
        if ($bVisibilityCheck)
        {
          if ($oLayer->status == MS_ON || $oLayer->status == MS_DEFAULT)
          {
              if ($oLayer->template != "")
              {
                  $bQueryable = true;
              }
              else
              {
                  for( $j=0; $j < $oLayer->numclasses; $j++)
                  {
                      $oClass = $oLayer->getClass($j);
                      if ($oClass->template != "")
                      {
                          $bQueryable = true;
                          break;
                      }
                  }
              }
          }
        }
        else
        {
            if ($oLayer->template != "")
            {
                $bQueryable = true;
            }
            else
            {
                for( $j=0; $j < $oLayer->numclasses; $j++)
                {
                    $oClass = $oLayer->getClass($j);
                    if ($oClass->template != "")
                    {
                        $bQueryable = true;
                        break;
                    }
                }
            }
        }

        // only add if on
        if ( $bQueryable )
        {
            array_push( $aszLayers, $oLayer->name );
        }
    }

    $aResults = array();
    // Check if there's any queryable layers
    if(count($aszLayers) >= 1)
    {
        // commit
        $oMapQuery->setQueryLayersByName( $aszLayers );

        // execute query
        $oResultSet = $oMapQuery->executePointQuery( $nMinX, $nMinY );

        // Check if there's any result
        if($oResultSet)
        {
            $nCount = $oResultSet->nNumResults;

            for ( $i=0; $i<$nCount; $i++ )
            {
                // process each result set
                $aRows = $oResultSet->getResultSet( $i );
                $oLayer = $oMapSession->oMap->getLayerByName( $aszLayers[$i] );

                /*
                 * THIS IS FOR LOCAL LAYER RESULTS
                 */
                // check if single or multiple results were returned
                if ( is_array( $aRows ) )
                {
                    $aResults[$oLayer->index] = $aRows;
                }
                else
                {
                    // check if no results on local layer
                    if ( strcasecmp(substr( $aRows, 0, 26 ), "no results found for layer" ) != 0 )
                    {
                        /*
                         * THIS IS FOR WMS RESULTS
                         */
                        $aResults[$oLayer->index] = processWMSGetFeatureInfo( $aRows );

                    }
                }
            }
        }
    }
    return $aResults;
}

function processWMSGetFeatureInfo( $szURL )
{
    $aResults = array();
   //try to determine the appropriate format to use based on the server
    if ( stristr( $szURL, "cubeserv" ) !== false ) //CubeWerx servers
    {
        $szFormat = "application/x-cubestor-gml.1";
        $szURL = setURLQueryParam( $szURL, "INFO_FORMAT", $szFormat );
    }
    elseif (stristr( $szURL, "interceptor") !== false ) // possibly a CTIS server
    {
        $szFormat = "application/x-cubestor-gml.1";
        $szURL = setURLQueryParam( $szURL, "INFO_FORMAT", $szFormat );
    }
    else //possibly a mapserver server
    {
        $szFormat = "application/vnd.ogc.gml";
        $szURL = setURLQueryParam( $szURL, "INFO_FORMAT", $szFormat );
    }
    $aszRawFile = "";
    $nResult = getFeatureInfoResults( $szURL, $aszRawFile );
    if ($nResult == -1)
    {
        return $aResults; //failed to open the url at all

    }
    elseif ($nResult == 0) //unsupported format, try text/html
    {
        //echo "unsupported format ($szFormat)";
        $szFormat = "text/html";
        $szURL = setInfoFormatURL( $szURL, $szFormat );
        $nResult = getFeatureInfoResults( $szURL, $aszRawFile );

        if ($nResult != 1) //if text/html fails, return an error
            return $aResults;
    }
    //we got a result in szFormat.  Process accordingly.
    if (strcasecmp( $szFormat, "MIME" ) == 0  || strcasecmp( $szFormat, "text/html" ) == 0)
    {
        $aResults = implode( "", $aszRawFile);
    }
    elseif (strcasecmp( $szFormat, "application/x-cubestor-gml.1" ) == 0)  //cubestor gml
    {
        $szResult = "";
        $szXMLDoc = implode( "", $aszRawFile );
        if ($szXMLDoc != "")
        {
            $oXMLObj = new XMLObject( $szXMLDoc );
            foreach( $oXMLObj->children as $oResult )
            {
                if (strcasecmp( $oResult->name, "FeatureCollection" ) == 0 )
                    $aResults = processCubeWerxGML( $oResult );
            }
        }
    }
    elseif( strcasecmp( $szFormat, "application/vnd.ogc.gml" ) == 0 )  //mapserver gml
    {
        $szResult = "";
        $szXMLDoc = implode( "", $aszRawFile);
        if ($szXMLDoc != "")
        {
            $oXMLObj = new XMLObject( $szXMLDoc );
            foreach( $oXMLObj->children as $oResult )
            {
                if (strcasecmp( $oResult->name, "msGMLOutput" ) == 0 )
                    $aResults = processMapServerGML( $oResult );
            }
        }
    }
    return $aResults;
}

function getFeatureInfoResults($szURL, &$aszRawFile)
{
    global $oMLT;
  //echo $szURL;

    $aszRawFile = @file( $szURL );

    if ($aszRawFile === false) // failed to open the URL entirely
        return -1;
    else
    {
        $szXMLDoc = trim(implode( "", $aszRawFile));

        //if the doc string is empty, fail.
        if ($szXMLDoc == "")
        {
            //echo "failing on empty doc string<BR>";
            return 0;
        }

        //test if it is an xml document
        if (stristr($szXMLDoc, "<?xml") === false)
        {
            //its not XML, that's okay.
            //echo "not an XML doc<BR>";
            return 1;
        }

        //try to parse it as XML
        $oXMLObj = new XMLObject( $szXMLDoc );

        //no children in base object means there where no XML tags in the doc,
        if (count( $oXMLObj->children ) == 0)
        {
            // No children, that can happen.
            //echo "zero children<BR>";
            return 1;
        }
        if (strcasecmp($oXMLObj->children[0]->name, "ServiceException") == 0)
        {
            //a WMS Service Exception, return -1.
            $oXMLObj->dumpObject();
            //echo "service exception<BR>";
            return -1;
        }

        //otherwise the document may be okay
        return 1;
    }
}

//generic function to set a single value in a URL query string
//and return the whole URL back again.
function setURLQueryParam( $szURL, $szValue, $szParam )
{
    global $oMLT;

    $aszParts = parse_url( $szURL );
    parse_str( $aszParts["query"], $aszQuery );
    $aszQuery[$szValue] = $szParam;
    $szQuery = "";
    $szSep = "";
    foreach( $aszQuery as $key => $value )
    {
        $szQuery .= $szSep.$key."=".urlencode( $value );
        $szSep = "&";
    }
    $aszParts["query"] = $szQuery;
    return glue_url( $aszParts );
}

//generic function to glue a URL back together after it has been parsed by
//parse_url (taken from http://www.php.net/parse_url with some modifications)
function glue_url($parsed)
{
    if (! is_array($parsed))
        return false;
    $uri = (isset($parsed['scheme'])) ? $parsed['scheme'].':'.((strtolower($parsed['scheme']) == 'mailto') ? '':'//'): '';
    $uri .= (isset($parsed['user'])) ? $parsed['user'].($parsed['pass']? ':'.$parsed['pass']:'').'@':'';
    $uri .= (isset($parsed['host'])) ? $parsed['host'] : '';
    $uri .= (isset($parsed['port'])) ? ':'.$parsed['port'] : '';
    $uri .= (isset($parsed['path'])) ? $parsed['path'] : '';
    $uri .= (isset($parsed['query'])) ? '?'.$parsed['query'] : '';
    $uri .= (isset($parsed['fragment'])) ? '#'.$parsed['fragment'] : '';
    return $uri;
}

//process CubeWerx GML output from a GetFeatureInfo and return formatted
//results
function processCubeWerxGML( $oXMLObj )
{
    $aResults = array();

    foreach($oXMLObj->children as $oMember)
    {
        if (strcasecmp($oMember->name, "featureMember") != 0)
            continue;

        $nFeatureMembers ++;

        $nFeatures  = 0;
        foreach( $oMember->children as $oFeature )
        {
            if (strcasecmp($oFeature->name, "feature" ) != 0)
                continue;

            $aSingleResult = array();

            foreach( $oFeature->children as $oProperty )
            {
                if (strcasecmp($oProperty->name, "property") != 0)
                    continue;

                $szTypeName = trim((isset($oProperty->typename)) ?
                              $oProperty->typename : trim($oMLT->get("6", "missing typename")));
                $szValue = trim((isset($oProperty->value)) ? $oProperty->value : trim($oMLT->get("9", "no value")) );

                $aSingleResult[$szTypeName] = $szValue;
            }
            $aResults[] = $aSingleResult;
        }
    }
    return $aResults;
}


//process MapServer GML output from a GetFeatureInfo and return formatted
//results
function processMapServerGML( $oXMLObj )
{
    $aResults = array();
    foreach($oXMLObj->children as $oLayer)
    {
        $aSingleResult = array();
        foreach( $oLayer->children as $oFeature )
        {
            foreach( $oFeature->children as $oProperty )
            {
                $szAttributeName = trim($oProperty->name);
                $szValue = trim($oProperty->value);
                $aSingleResult[$szAttributeName] = $szValue;
            }
        }
        $aResults[] = $aSingleResult;
    }
    return $aResults;
}
?>