<?php
/**
 * UI Template Parser
 *
 * @project     CWC2
 * @revision    $Id: TemplateParser.php,v 1.8 2003/12/11 00:47:08 pspencer Exp $
 * @purpose     Parse UI Template HTML file
 * @author      DM Solutions Group (assefa@dmsolutions.ca,spencer@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */



/**
 * TemplateParser provides parsing functions for extracting a set of elements
 * from a text file and substituting them back into the text file after some
 * arbitrary manipulation.
 *
 * Typical useage might be:
 *
 * $oParser = new TemplateParser()
 * $oParser->SetPattern( $myPattern );
 * $oParser->LoadTemplate( $myTemplate );
 * $nElements = $oParser->Parse();
 * for ($i=0; $i<$nElements; $i++)
 * {
 *    $szElement = $oParser->GetElement( $i );
 *    $szElement = "My Element ->".$szElement." <- My Element";
 *    $oParser->SetElement( $i, $szElement );
 * }
 * $oParser->ProcessElements();
 * echo $oParser->GetTemplate();
 *
 * this would replace all elements that match $myPattern with the element
 * wrapped in some text
 */
 class TemplateParser
{
    /**
     * the contents of the template before, during and after parsing and
     * elements substitution
     */
    var $mszTemplate;

    /**
     * array of elements found by the parser
     */
    var $maszElements;

    /**
     * array of CWC placeholders
     */
    var $maszPlaceholders;

    /**
     * the RegExp pattern to use for finding elements
     */
    var $mszPattern;

    function TemplateParser()
    {
        $this->maszElements = array();
        $this->maszPlaceholders = array();
        $this->mszPattern = "";
    }

    /**
     * Set the template parsing pattern, a regular expression pattern
     * compatible with preg_replace_callback.
     * @param $szPattern the pattern to use
     */
    function SetPattern( $szPattern )
    {
        $this->mszPattern = $szPattern;
    }

    /**
     * Load a template into the parser.  This prepares the file for parsing
     * but doesn't actually parse it.
     * @param $szTemplateName the filename of the template to parse
     * @return boolean true if the file loaded, false if an error happened
     */
    function LoadTemplate(&$szTemplate)
    {
        $this->mszTemplate =& $szTemplate;
    }

    /**
     * parse the current template contents into a set of elements
     */
    function Parse()
    {
        $oApp =& GetChameleonApplication();

        // First, include all files to include.
        $nPos = strpos($this->mszTemplate, "[#");
        while ($nPos !== false)
        {
            $nEndPos = strpos($this->mszTemplate, "#]");

            if ($nEndPos === false)
                break;

            $szTemplateName = substr($this->mszTemplate, $nPos+2, $nEndPos-$nPos-2);

            // Make a copy of the current teplate
            $szTemplate = $this->mszTemplate;

            $oApp->CWCLoadTemplate($szTemplateName);

            $this->mszTemplate =& str_replace("[#$szTemplateName#]", $oApp->mszTemplate, $szTemplate);

            $nPos = strpos($this->mszTemplate, "[#");
        }

        // Then, replace all [$something$] to the correct url var value
        $nPos = strpos($this->mszTemplate, "[$");
        while($nPos !== false)
        {
            $nEndPos = strpos($this->mszTemplate, "$]", $nPos);
            $szTag = substr($this->mszTemplate, $nPos, $nEndPos-$nPos+2);
            $szVal = strtoupper(substr($szTag, 2, -2));
            if (isset($oApp->moHttpFormVars->maszVars[$szVal]))
            {
                $this->mszTemplate =& str_replace($szTag, 
                        $oApp->moHttpFormVars->maszVars[$szVal],
                        $this->mszTemplate);
            }   
            $nPos = strpos($this->mszTemplate, "[$", $nPos+2);
        }

        if (strlen($this->mszTemplate) == 0)
            return 0;

        $this->maszElements = array();
        $this->maszPlaceholders = array();

        $this->mszTemplate = preg_replace_callback( $this->mszPattern,
                    array( &$this, "ParseCallback" ), $this->mszTemplate );

        return $this->NumElements();
    }

    /**
     * parser callback function to handle each tag as it is found
     * @param $aszMatch an array of matched parameters, use index 0
     * @return string the value to replace the tag with
     */
    function ParseCallback( $aszMatch )
    {
        $nID = count( $this->maszElements );
        $this->maszElements[$nID] = $aszMatch[0];
        $this->maszPlaceholders[$nID] = "<TemplateParser ID=\"$nID\"/>";
        return $this->maszPlaceholders[$nID];
    }

    /**
     * process widget tags back into the template
     */
    function ProcessElements()
    {
        $this->mszTemplate = str_replace( $this->maszPlaceholders,
                                $this->maszElements, $this->mszTemplate );
        //$max = count( $this->maszElements );
        //for ($i=0;$i<$max;$i++)
        //{
        //    $this->mszTemplate = str_replace( $this->maszPlaceholders[$i], $this->maszElements[$i], $this->mszTemplate );
        //}

    }

    /**
     * process arbitrary pattern replacements in the template
     * @param $szPattern the preg_replace compatible pattern to search for
     * @param $szReplacement the string to put in place of the pattern
     * @param $nIndex Index of the pattern instance to replace. -1 mean ALL, 0 mean NONE, greater than 0 is the index.
     * @see http://www.php.net/preg_replace
     */
    function ProcessPatternReplacement( $szPattern, $szReplacement, $nIndex=-1 )
    {
        $GLOBALS['nPPRCnt'] = 0;
        $GLOBALS['nPPRIndex'] = $nIndex;
        $GLOBALS['szPPRReplacement'] = $szReplacement;

        if ($nIndex == -1)
            $this->mszTemplate = preg_replace( $szPattern, $szReplacement, $this->mszTemplate );
    else
            $this->mszTemplate = preg_replace_callback( $szPattern, array(&$this, "preg_callback"),
                                                        $this->mszTemplate);
    }
 
    function preg_callback($matches)
    {
        $GLOBALS['nPPRCnt']++;

        if( $GLOBALS['nPPRCnt'] == $GLOBALS['nPPRIndex'])
            return $matches[0].$GLOBALS['szPPRReplacement'];
        else
            return $matches[0];
    }

    /**
     * process arbitrary replacements in the template
     * @param $szSearch the string to search for
     * @param $szReplacement the string to put in place of the search string
     */
    function ProcessStringReplacement( $szSearch, $szReplacement )
    {
        $this->mszTemplate = str_replace( $szSearch, $szReplacement, $this->mszTemplate );
    }

    /**
     * return the number of widget tags
     * @return int the number of widget tags
     */
    function NumElements()
    {
        return count( $this->maszElements );
    }

    /**
     * get an element by ID
     * @param $nID the id of the element to get
     * @return string the element at index $nID
     */
    function GetElement( $nID )
    {
        return $this->maszElements[$nID];
    }

    /**
     * set an element by ID
     * @param $nID the id of the element to set
     * @param $szElement the text to set for the element
     */
    function SetElement( $nID, $szElement )
    {
        $this->maszElements[$nID] = $szElement;
    }

}

class CWCTemplateParser extends TemplateParser
{
    function CWCTemplateParser()
    {
        $this->mszPattern = "/(<Cwc2[^>]*\/>)|(<Cwc2[^>]*>.*<\/CWC2>)/iUs";
    }
}
?>
